@@ -17,11 +17,11 @@
 #' @param patm Atmospheric pressure (Pa). When provided, overrides
 #'  \code{elv}, otherwise \code{patm} is calculated using standard
 #'  atmosphere (101325 Pa), corrected for elevation (argument \code{elv}),
-#'  using the function \link{patm}.
+#'  using the function \link{calc_patm}.
 #' @param elv Elevation above sea-level (m.a.s.l.). Is used only for 
 #'  calculating atmospheric pressure (using standard atmosphere (101325 Pa),
 #'  corrected for elevation (argument \code{elv}), using the function
-#' \link{patm}), if argument \code{patm} is not provided. If argument
+#' \link{calc_patm}), if argument \code{patm} is not provided. If argument
 #' \code{patm} is provided, \code{elv} is overridden.
 #' @param kphio Apparent quantum yield efficiency (unitless). Defaults to
 #'  0.081785 for \code{method_jmaxlim="wang17", do_ftemp_kphio=TRUE, 
@@ -30,18 +30,23 @@
 #'  \code{method_jmaxlim="wang17", do_ftemp_kphio=FALSE, do_soilmstress=FALSE},
 #'  corresponding to the empirically fitted value as presented in Stocker et al.
 #'  (2019) Geosci. Model Dev. for model setup 'BRC', 'FULL', and 'ORG' 
-#'  respectively.
-#' @param beta Unit cost ratio. Defaults to 146.0 (see Stocker et al., 2019).
+#'  respectively, corresponding to \eqn{(a_L b_L)/4} in 
+#'  Eq.20 in Stocker et al. (2020) for C3 photosynthesis. For C4 photosynthesis
+#'  (\code{c4 = TRUE}), \code{kphio} defaults to 1.0, corresponding to the 
+#'   parametrisation by  Cai & Prentice (2020).
+#' @param beta Unit cost ratio. Defaults to 146.0 (see Stocker et al., 2019) for
+#'   C3 plants and 146/9 for C4 plants.
 #' @param soilm (Optional, used only if \code{do_soilmstress==TRUE}) Relative 
 #'  soil moisture as a fraction of field capacity (unitless). Defaults to 1.0 
 #'  (no soil moisture stress). This information is used to calculate
-#'  an empirical soil moisture stress factor (\link{soilmstress}) whereby
+#'  an empirical soil moisture stress factor (\link{calc_soilmstress}) whereby
 #'  the sensitivity is determined by average aridity, defined by the local 
 #'  annual mean ratio of actual over potential evapotranspiration, supplied by
 #'  argument \code{meanalpha}.
 #' @param meanalpha (Optional, used only if \code{do_soilmstress==TRUE}) Local 
 #'  annual mean ratio of actual over potential evapotranspiration, measure for 
-#'  average aridity. Defaults to 1.0.
+#'  average aridity. Defaults to 1.0. Only scalar numbers are accepted. If 
+#'  a vector is provided, only the first element will be used.
 #' @param apar_soilm (Optional, used only if \code{do_soilmstress==TRUE}) 
 #'  Parameter determining the sensitivity of the empirical soil moisture stress 
 #'  function. Defaults to 0.0, the empirically fitted value as presented in 
@@ -54,23 +59,22 @@
 #'  Stocker et al. (2019) Geosci. Model Dev. for model setup 'FULL' 
 #'  (corresponding to a setup with \code{method_jmaxlim="wang17", 
 #'  do_ftemp_kphio=TRUE, do_soilmstress=TRUE}).
-#' @param c4 (Optional) A logical value specifying whether the C3 or C4 
+#' @param c4 (Optional) A logical value specifying whether the C3 or C4
 #'  photosynthetic pathway is followed.Defaults to \code{FALSE}. If \code{TRUE},
-#'  the leaf-internal CO2 concentration is assumed to be very large and 
-#'  \eqn{m} (returned variable \code{mj}) tends to 1, and \eqn{m'} tends to 
-#'  0.669 (with \code{c = 0.41}).
-#' @param method_optci (Optional) A character string specifying which method is
-#'  to be used for calculating optimal ci:ca. Defaults to \code{"prentice14"}.
-#'  Available also \code{"prentice14_num"} for a numerical solution to the same
-#'  optimization criterium as used for \code{"prentice14"}.
+#'  the leaf-internal CO2 concentration is still estimated using beta but
+#'  \eqn{m} (returned variable \code{mj}) tends to 1, and \eqn{m'} tends to
+#'  0.669 (with \code{c = 0.41}) to represent CO2 concentrations within the leaf.
+#'  With \code{do_ftemp_kphio = TRUE}, a C4-specific temperature dependence of
+#'  the quantum yield efficiency is used (see \link{ftemp_kphio}).
 #' @param method_jmaxlim (Optional) A character string specifying which method 
 #'  is to be used for factoring in Jmax limitation. Defaults to \code{"wang17"},
 #'  based on Wang Han et al. 2017 Nature Plants and (Smith 1937). Available is 
 #'  also \code{"smith19"}, following the method by Smith et al., 2019 Ecology 
 #'  Letters, and \code{"none"} for ignoring effects of Jmax limitation.
 #' @param do_ftemp_kphio (Optional) A logical specifying whether 
-#'  temperature-dependence of quantum yield efficiency after Bernacchi et al., 
-#'  2003 is to be accounted for. Defaults to \code{TRUE}.
+#'  temperature-dependence of quantum yield efficiency is used. See \link{ftemp_kphio}
+#'  for details. Defaults to \code{TRUE}. Only scalar numbers are accepted. If 
+#'  a vector is provided, only the first element will be used.
 #' @param do_soilmstress (Optional) A logical specifying whether an empirical 
 #' soil moisture stress factor is to be applied to down-scale light use 
 #' efficiency (and only light use efficiency). Defaults to \code{FALSE}.
@@ -87,10 +91,10 @@
 #'  \item \code{ca}: Ambient CO2 expressed as partial pressure (Pa)
 #'  
 #'  \item \code{gammastar}: Photorespiratory compensation point \eqn{\Gamma*}, 
-#'   (Pa), see \link{gammastar}.
+#'   (Pa), see \link{calc_gammastar}.
 #'  
 #'  \item \code{kmm}: Michaelis-Menten coefficient \eqn{K} for photosynthesis 
-#'  (Pa), see \link{kmm}.
+#'  (Pa), see \link{calc_kmm}.
 #'  
 #'  \item \code{ns_star}: Change in the viscosity of water, relative to its 
 #'   value at 25 deg C (unitless).
@@ -108,8 +112,8 @@
 #'    \xi = \sqrt (\beta (K+ \Gamma*) / (1.6 \eta*))
 #'   }
 #'   \eqn{\beta} is given by argument \code{beta}, \eqn{K} is 
-#'   \code{kmm} (see \link{kmm}), \eqn{\Gamma*} is 
-#'   \code{gammastar} (see \link{gammastar}). \eqn{\eta*} is \code{ns_star}.
+#'   \code{kmm} (see \link{calc_kmm}), \eqn{\Gamma*} is 
+#'   \code{gammastar} (see \link{calc_gammastar}). \eqn{\eta*} is \code{ns_star}.
 #'   \eqn{D} is the vapour pressure deficit (argument \code{vpd}), \eqn{ca} is 
 #'   the ambient CO2 partial pressure in Pa (\code{ca}).
 #'   
@@ -129,17 +133,17 @@
 #'                         with \eqn{c=0.41} (Wang et al., 2017) if \code{method_jmaxlim=="wang17"}. \eqn{Mc} is
 #'                         the molecular mass of C (12.0107 g mol-1). \eqn{m} is given returned variable \code{mj}.
 #'                         If \code{do_soilmstress==TRUE}, \eqn{LUE} is multiplied with a soil moisture stress factor,
-#'                         calculated with \link{soilmstress}.
+#'                         calculated with \link{calc_soilmstress}.
 #'         \item \code{mj}: Factor in the light-limited assimilation rate function, given by
 #'                         \deqn{
 #'                             m = (ci - \Gamma*) / (ci + 2 \Gamma*)
 #'                        }
-#'                        where \eqn{\Gamma*} is given by \code{gammastar}.
+#'                        where \eqn{\Gamma*} is given by \code{calc_gammastar}.
 #'         \item \code{mc}: Factor in the Rubisco-limited assimilation rate function, given by
 #'                         \deqn{
 #'                             mc = (ci - \Gamma*) / (ci + K)
 #'                        }
-#'                        where \eqn{K} is given by \code{kmm}.
+#'                        where \eqn{K} is given by \code{calc_kmm}.
 #'         \item \code{gpp}: Gross primary production (g C m-2), calculated as
 #'                        \deqn{
 #'                            GPP = Iabs LUE
@@ -189,12 +193,16 @@
 #'   
 #'  \item \code{omega_star}: Term corresponding to \eqn{\omega^\ast}, defined by
 #'   Eq. 18 in Smith et al. (2019), and Eq. E21 in Stocker et al. (2019).
-#'  }
+#'  }patm
 #'
 #' @references  
 #'  Bernacchi, C. J., Pimentel, C., and Long, S. P.:  In vivo temperature response func-tions  of  parameters
 #'  required  to  model  RuBP-limited  photosynthesis,  Plant  Cell Environ., 26, 1419–1430, 2003
 #'
+#   Cai, W., and Prentice, I. C.: Recent trends in gross primary production 
+#'  and their drivers: analysis and modelling at flux-site and global scales,
+#'  Environ. Res. Lett. 15 124050 https://doi.org/10.1088/1748-9326/abc64e, 2020
+#
 #'  Heskel,  M.,  O’Sullivan,  O.,  Reich,  P.,  Tjoelker,  M.,  Weerasinghe,  L.,  Penillard,  A.,Egerton, J.,
 #'  Creek, D., Bloomfield, K., Xiang, J., Sinca, F., Stangl, Z., Martinez-De La Torre, A., Griffin, K.,
 #'  Huntingford, C., Hurry, V., Meir, P., Turnbull, M.,and Atkin, O.:  Convergence in the temperature response
@@ -243,14 +251,18 @@ rpmodel <- function(
   ppfd,
   patm = NA,
   elv = NA,
-  kphio = ifelse(do_ftemp_kphio, ifelse(do_soilmstress, 0.087182, 0.081785), 0.049977),
-  beta = 146.0,
+  kphio = ifelse(c4, 1.0,
+                 ifelse(do_ftemp_kphio,
+                        ifelse(do_soilmstress,
+                               0.087182,
+                               0.081785),
+                        0.049977)),
+  beta = ifelse(c4, 146/9, 146),
   soilm = stopifnot(!do_soilmstress),
   meanalpha = 1.0,
   apar_soilm = 0.0,
   bpar_soilm = 0.73300,
   c4 = FALSE,
-  method_optci = "prentice14",
   method_jmaxlim = "wang17",
   do_ftemp_kphio = TRUE,
   do_soilmstress = FALSE,
@@ -260,10 +272,20 @@ rpmodel <- function(
 
   # Check arguments
   if (identical(NA, elv) && identical(NA, patm)){
-    stop("Aborted. Provide either elevation (arugment elv) or atmospheric pressure (argument patm).")
+    stop(
+    "Aborted. Provide either elevation (arugment elv) or 
+     atmospheric pressure (argument patm)."
+    )
   } else if (!identical(NA, elv) && identical(NA, patm)){
-    if (verbose) warning("Atmospheric pressure (patm) not provided. Calculating it as a function of elevation (elv), assuming standard atmosphere (101325 Pa at sea level).")
-    patm <- patm(elv)
+    if (verbose) {
+      warning(
+      "Atmospheric pressure (patm) not provided. Calculating it as a
+      function of elevation (elv), assuming standard atmosphere 
+      (101325 Pa at sea level)."
+      )
+    }
+    
+    patm <- calc_patm(elv)
   }
 
   #---- Fixed parameters--------------------------------------------------------
@@ -275,15 +297,25 @@ rpmodel <- function(
   #---- Temperature dependence of quantum yield efficiency----------------------
   ## 'do_ftemp_kphio' is not actually a stress function, but is the temperature-dependency of
   ## the quantum yield efficiency after Bernacchi et al., 2003 PCE
-  if (do_ftemp_kphio){
-    ftemp_kphio <- ftemp_kphio( tc, c4 )
+  if (length(do_ftemp_kphio) > 1){
+    warning("Argument 'do_ftemp_kphio' has length > 1. Only the first element is used.")
+    do_ftemp_kphio <- do_ftemp_kphio[1]
+  }
+  if (do_ftemp_kphio) {
+    kphio <- ftemp_kphio( tc, c4 ) * kphio
   } else {
-    ftemp_kphio <- 1.0
+    if (c4){
+      kphio <- ftemp_kphio( 15.0, c4 ) * kphio
+    }
   }
 
   #---- soil moisture stress as a function of soil moisture and mean alpha -----
   if (do_soilmstress) {
-    soilmstress <- soilmstress( soilm, meanalpha, apar_soilm, bpar_soilm )
+    if (length(meanalpha) > 1){
+      warning("Argument 'meanalpha' has length > 1. Only the first element is used.")
+      meanalpha <- meanalpha[1]
+    }
+    soilmstress <- calc_soilmstress( soilm, meanalpha, apar_soilm, bpar_soilm )
   }
   else {
     soilmstress <- 1.0
@@ -294,35 +326,20 @@ rpmodel <- function(
   ca <- co2_to_ca( co2, patm )
 
   ## photorespiratory compensation point - Gamma-star (Pa)
-  gammastar <- gammastar( tc, patm )
+  gammastar <- calc_gammastar( tc, patm )
 
   ## Michaelis-Menten coef. (Pa)
-  kmm <- kmm( tc, patm )   ## XXX Todo: replace 'NA' here with 'patm'
+  kmm <- calc_kmm( tc, patm )
 
   ## viscosity correction factor = viscosity( temp, press )/viscosity( 25 degC, 1013.25 Pa)
-  ns      <- viscosity_h2o( tc, patm )  # Pa s
+  ns      <- viscosity_h2o( tc, patm )  # Pa sc4, 1.0,
   ns25    <- viscosity_h2o( kTo, kPo )  # Pa s
   ns_star <- ns / ns25  # (unitless)
 
   ##----Optimal ci -------------------------------------------------------------
   ## The heart of the P-model: calculate ci:ca ratio (chi) and additional terms
+  out_optchi <- optimal_chi( kmm, gammastar, ns_star, ca, vpd, beta, c4 )
   
-  if (c4){
-
-    # "dummy" ci:ca for C4 plants
-    out_optchi <- chi_c4()
-
-  } else if (method_optci=="prentice14"){
-
-    #---- Full formualation (Gamma-star not zero), analytical solution ---------
-    out_optchi <- optimal_chi( kmm, gammastar, ns_star, ca, vpd, beta )
-
-  } else {
-
-    stop("rpmodel(): argument method_optci not idetified.")
-
-  }
-
   ## leaf-internal CO2 partial pressure (Pa)
   ci <- out_optchi$chi * ca
 
@@ -336,7 +353,6 @@ rpmodel <- function(
 
     out_lue_vcmax <- lue_vcmax_c4(
       kphio,
-      ftemp_kphio,
       c_molmass,
       soilmstress
       )
@@ -347,7 +363,6 @@ rpmodel <- function(
     out_lue_vcmax <- lue_vcmax_wang17(
       out_optchi,
       kphio,
-      ftemp_kphio,
       c_molmass,
       soilmstress
       )
@@ -357,7 +372,6 @@ rpmodel <- function(
     out_lue_vcmax <- lue_vcmax_smith19(
       out_optchi,
       kphio,
-      ftemp_kphio,
       c_molmass,
       soilmstress
       )
@@ -367,7 +381,6 @@ rpmodel <- function(
     out_lue_vcmax <- lue_vcmax_none(
       out_optchi,
       kphio,
-      ftemp_kphio,
       c_molmass,
       soilmstress
       )
@@ -388,54 +401,59 @@ rpmodel <- function(
   rd_unitiabs  <- rd_to_vcmax * (ftemp_inst_rd / ftemp25_inst_vcmax) * out_lue_vcmax$vcmax_unitiabs
 
   #---- Quantities that scale linearly with absorbed light ---------------------
-  # len <- length(out_lue_vcmax[[1]])
   iabs <- fapar * ppfd
 
   # Gross Primary Productivity
-  # gpp <- ifelse(any(is.na(iabs)),
-  # rep(NA, len), iabs * out_lue_vcmax$lue)   # in g C m-2 s-1
   gpp <- iabs * out_lue_vcmax$lue   # in g C m-2 s-1
 
   # Vcmax per unit ground area is the product of the intrinsic quantum
   # efficiency, the absorbed PAR, and 'n'
-  # vcmax <- ifelse(any(is.na(iabs)), rep(NA, len), iabs * out_lue_vcmax$vcmax_unitiabs)
   vcmax <- iabs * out_lue_vcmax$vcmax_unitiabs
 
   ## (vcmax normalized to 25 deg C)
-  # vcmax25 <- ifelse(any(is.na(iabs)), rep(NA, len), iabs * vcmax25_unitiabs)
   vcmax25 <- iabs * vcmax25_unitiabs
 
   ## Dark respiration
-  # rd <- ifelse(!is.na(iabs), iabs * rd_unitiabs, rep(NA, len))
   rd <- iabs * rd_unitiabs
 
-  # Jmax using again A_J = A_C
-  # fact_jmaxlim <- ifelse(!is.na(iabs),
-  #                        vcmax * (ci + 2.0 * gammastar) / (kphio * iabs * (ci + kmm)),
-  #                        rep(NA, len))
+  # Jmax using again A_J = A_C, derive the "Jmax limitation factor" 
   fact_jmaxlim <- vcmax * (ci + 2.0 * gammastar) / (kphio * iabs * (ci + kmm))
-
-  # jmax <- ifelse(!is.na(iabs),
-  #                4.0 * kphio * iabs / sqrt( (1.0/fact_jmaxlim)**2 - 1.0 ),
-  #                rep(NA, len))
+  
+  # use definition of Jmax limitation factor (L in Eq. 13) and solve for Jmax.
   jmax <- 4.0 * kphio * iabs / sqrt( (1.0/fact_jmaxlim)^2 - 1.0 )
-
+  
+  # ## Alternatively, Jmax can be calculated from Eq. F10 in Stocker et al., 2020
+  # kc <- 0.41
+  # jmax_alt <- 4.0 * kphio * iabs * sqrt((out_optchi$mj / kc)^(2/3) - 1.0)
+  # fact_jmaxlim_alt <- 1.0 / sqrt(1 + (4.0 * kphio * iabs / jmax_alt)^2)
+  
   ftemp25_inst_jmax <- ftemp_inst_jmax( tc, tc, tcref = 25.0 )
   jmax25 <- jmax / ftemp25_inst_jmax
 
   ## Test: at this stage, verify if A_J = A_C
-  a_j <- kphio * iabs * (ci - gammastar)/(ci + 2 * gammastar) * fact_jmaxlim
-  a_c <- vcmax * (ci - gammastar) / (ci + kmm)
-  if (any(abs(a_j/a_c - 1) > 0.001)){
-    stop("rpmodel(): light and Rubisco-limited assimilation rates
-                 are not identical.")
-  } 
+  if (c4){
+    a_j = kphio * iabs * out_optchi$mj * fact_jmaxlim
+    a_c = vcmax * out_optchi$mc
+  } else {
+    a_j <- kphio * iabs * (ci - gammastar)/(ci + 2.0 * gammastar) * fact_jmaxlim
+    a_c <- vcmax * (ci - gammastar) / (ci + kmm)
+  }
+  
+  a_j_eq_a_c <- all.equal(a_j, a_c, tol = 0.001)
+  if (! isTRUE(a_j_eq_a_c)) {
+    warning("rpmodel(): light and Rubisco-limited assimilation rates ",
+            "are not identical.\n", a_j_eq_a_c)
+  }
 
   # Assimilation is not returned because it should not be confused with what 
   # is usually measured should use instantaneous assimilation for comparison to
   # measurements. This is returned by inst_rpmodel().
   assim <- ifelse(a_j < a_c , a_j, a_c)
-  if (any(abs(assim - gpp / c_molmass) > 0.001)) stop("rpmodel(): Assimilation and GPP are not identical.")
+  assim_eq_check <- all.equal(assim, gpp / c_molmass, tol = 0.001)
+  if (! isTRUE(assim_eq_check)) {
+      warning("rpmodel(): Assimilation and GPP are not identical.\n",
+              assim_eq_check)
+  }
 
   ## average stomatal conductance
   gs <- assim / (ca - ci)
